/**
 * @file
 *
 * @ingroup rtems_bsd_rtems
 *
 * @brief TODO.
 */

/*
 * Copyright 2020 Chris Johns. All Rights Reserved.
 *
 *  Contemporary Software
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <machine/rtems-bsd-kernel-space.h>

#include <sys/param.h>
#include <sys/filedesc.h>
#include <sys/proc.h>

#include <rtems/libio.h>

#include <machine/rtems-bsd-libio.h>

rtems_libio_t*
rtems_bsd_libio_iop_allocate(void)
{
  rtems_libio_t *iop = rtems_libio_allocate();
  if (iop != NULL) {
    iop->pathinfo.mt_entry = &rtems_filesystem_null_mt_entry;
    rtems_filesystem_location_add_to_mt_entry(&iop->pathinfo);
  }
  return iop;
}

int
rtems_bsd_libio_iop_allocate_with_file(struct thread *td,
                                       int fd,
                                       const rtems_filesystem_file_handlers_r* ops)
{
  rtems_libio_t *iop = rtems_bsd_libio_iop_allocate();
  int iofd = -1;
  if (iop != NULL) {
    int error = rtems_bsd_libio_iop_set_bsd_fd(td, fd, iop, ops);
    /*
     * The fp is held and needs to be dropped and that drops the iop.
     */
    if (error == 0) {
      rtems_libio_iop_hold(iop);
      iofd = rtems_libio_iop_to_descriptor(iop);
    } else {
      rtems_libio_free(iop);
    }
  }
  return iofd;
}

int
rtems_bsd_libio_iop_set_bsd_fd(struct thread *td,
                               int fd,
                               rtems_libio_t *iop,
                               const rtems_filesystem_file_handlers_r* ops)
{
  struct filedesc* fdp;
  int error;
  fdp = td->td_proc->p_fd;
  if (fd < fdp->fd_nfiles) {
    struct file* fp = fdp->fd_ofiles[fd].fde_file;
    rtems_libio_iop_flags_set(iop,
                              LIBIO_FLAGS_OPEN |
                              rtems_bsd_libio_fflag_to_flags(fp->f_flag));
    if (ops != NULL)
      iop->pathinfo.handlers = ops;
    iop->pathinfo.node_access_2 = (intptr_t*) fd;
    rtems_bsd_libio_iop_set_bsd_descriptor(iop, fd);
    error = 0;
  }
  else {
    error = EBADF;
  }
  return error;
}

void
rtems_bsd_libio_loc_set_vnode(rtems_filesystem_location_info_t *loc,
                              struct vnode *vp)
{
  loc->node_access = vp;
}

void
rtems_bsd_libio_loc_set_vnode_parent(rtems_filesystem_location_info_t *loc,
                                     struct vnode *dvp)
{
  loc->node_access_2 = dvp;
}
