/*
20140117
Jan Mojzis
Public domain.
*/

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include "load.h"
#include "log.h"
#include "open.h"
#include "writeall.h"
#include "purge.h"
#include "global.h"
#include "bug.h"
#include "e.h"
#include "purge.h"
#include "readall.h"
#include "blocking.h"
#include "sshcrypto.h"
#include "subprocess.h"

/*
The 'subprocess_sign' function reads secret-key from 'keydir' and
signs the data 'x' of length 'xlen' and returns it in 'y'.
Caller is expecting 'y' of length 'ylen'. Signing is done
in a different process, so secret-key is in a separate
memory space than rest of the program.
*/
int subprocess_sign(unsigned char *y, long long ylen, const char *keydir, unsigned char *x, long long xlen) {

    pid_t pid;
    int status;

    unsigned char sk[sshcrypto_sign_SECRETKEYMAX];
    unsigned char sm[sshcrypto_sign_MAX + sshcrypto_hash_MAX];
    unsigned long long smlen;

    if (ylen != sshcrypto_sign_bytes) bug_inval();
    if (xlen != sshcrypto_hash_bytes) bug_inval();
    if (!y || !keydir || !x) bug_inval();

    /* signing starts here */
    if (chdir(keydir) == -1) {
        log_w2("sign: unable to change directory to ", keydir);
        global_die(111);
    }
    if (load(sshcrypto_sign_secretkeyfilename, sk, sshcrypto_sign_secretkeybytes) == -1) {
        log_w4("sign: unable to load secret-key from file ", keydir, "/", sshcrypto_sign_secretkeyfilename);
        purge(sk, sizeof sk);
        global_die(111);
    }
    if (sshcrypto_sign(sm, &smlen, x, sshcrypto_hash_bytes, sk) != 0) {
        log_w4("sign: unable to sign using secret-key from file ", keydir, "/", sshcrypto_sign_secretkeyfilename);
        purge(sk, sizeof sk);
        global_die(111);
    }
    purge(sk, sizeof sk);

    byte_copy(y, sshcrypto_sign_bytes, sm);

    return 0;
}
